/**
 * Copyright 2020 NEM
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
import { TransactionURI } from 'symbol-uri-scheme';
import { AggregateTransaction, MosaicInfo, PublicAccount, Transaction } from 'symbol-sdk';
import { Wallet } from 'symbol-hd-wallets';
import { AllowanceResult, BaseCommand, CommandOption, Context, TokenIdentifier, TokenPartition } from '../../../../index';
import { SecuritiesMetadata } from '../models/SecuritiesMetadata';
import { SecuritiesRestrictions } from '../models/SecuritiesRestrictions';
/**
 * @class NIP13.AbstractCommand
 * @package NIP13 Token Commands
 * @since v0.1.0
 * @description Abstract class that describes a token command for NIP13 compliant tokens.
 */
export declare abstract class AbstractCommand extends BaseCommand {
    /**
     * @description Execution context
     */
    readonly context: Context;
    /**
     * @description Token identifier
     */
    protected readonly identifier: TokenIdentifier;
    /**
     * @description The key provider used for the command execution.
     */
    keyProvider: Wallet;
    /**
     * @description The deterministic public account representing the token.
     */
    target: PublicAccount;
    /**
     * @description List of operators of said token.
     */
    operators: PublicAccount[];
    /**
     * @description Mosaic information (read from network).
     */
    mosaicInfo: MosaicInfo | undefined;
    /**
     * @description Partition records of said token.
     */
    partitions: TokenPartition[];
    /**
     * @description The securities metadata
     */
    metadata: SecuritiesMetadata | undefined;
    /**
     * @description The securities metadata
     */
    restrictions: SecuritiesRestrictions | undefined;
    /**
     * @description List of **required** arguments for this token command.
     */
    arguments: string[];
    /**
     * Construct a NIP13 command object around `context`
     * and `identifier`
     *
     * @param {Context} context
     */
    constructor(
    /**
     * @description Execution context
     */
    context: Context, 
    /**
     * @description Token identifier
     */
    identifier: TokenIdentifier, 
    /**
     * @description The key provider used for the command execution.
     */
    keyProvider: Wallet);
    /**
     * @description Getter for the command name.
     * @see {BaseCommand.name}
     **/
    abstract get name(): string;
    /**
     * Getter for the command descriptor.
     *
     * @return {string}
     **/
    abstract get descriptor(): string;
    /**
     * @description Build a command's transactions. Transactions returned here will
     *              be formatted to a transaction URI in the `execute()` step.
     * @see {BaseCommand.transactions}
     * @return {AggregateTransaction[]} Aggregate bonded transaction
     **/
    protected abstract get transactions(): Transaction[];
    /**
     * @description Method that verifies the allowance of an operator to
     *              execute the token command `TransferOwnership`.
     * @see {BaseCommand.canExecute}
     **/
    canExecute(actor: PublicAccount, argv?: CommandOption[]): AllowanceResult;
    /**
     * @description Method that executes the token command `TransferOwnership`.
     * @see {BaseCommand.execute}
     **/
    execute(actor: PublicAccount, argv?: CommandOption[]): TransactionURI;
    /**
     * @description Wrap the command's transactions inside an aggregate transaction.
     * @see {BaseCommand.wrap}
     * @return {AggregateTransaction[]} Aggregate bonded transaction
     **/
    protected prepare(): AggregateTransaction | Transaction;
}
