"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
/**
 * Copyright 2020 NEM
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
const symbol_uri_scheme_1 = require("symbol-uri-scheme");
const symbol_sdk_1 = require("symbol-sdk");
// internal dependencies
const index_1 = require("../../../../index");
const FailureEmptyContract_1 = require("../errors/FailureEmptyContract");
/**
 * @class NIP13.AbstractCommand
 * @package NIP13 Token Commands
 * @since v0.1.0
 * @description Abstract class that describes a token command for NIP13 compliant tokens.
 */
class AbstractCommand extends index_1.BaseCommand {
    /**
     * Construct a NIP13 command object around `context`
     * and `identifier`
     *
     * @param {Context} context
     */
    constructor(
    /**
     * @description Execution context
     */
    context, 
    /**
     * @description Token identifier
     */
    identifier, 
    /**
     * @description The key provider used for the command execution.
     */
    keyProvider) {
        super(context);
        this.context = context;
        this.identifier = identifier;
        this.keyProvider = keyProvider;
        /**
         * @description List of operators of said token.
         */
        this.operators = [];
        /**
         * @description Partition records of said token.
         */
        this.partitions = [];
        /**
         * @description List of **required** arguments for this token command.
         */
        this.arguments = [];
        this.target = this.identifier.target;
    }
    // end-region abstract methods
    /**
     * @description Method that verifies the allowance of an operator to
     *              execute the token command `TransferOwnership`.
     * @see {BaseCommand.canExecute}
     **/
    canExecute(actor, argv) {
        // validate mandatory inputs
        super.assertHasMandatoryArguments(argv, this.arguments);
        // by default, only operators can execute commands
        const isOperator = this.operators.some((p) => {
            return p.publicKey === actor.publicKey;
        });
        // allows only operators to transfer ownership tokens
        return new index_1.AllowanceResult(isOperator);
    }
    /**
     * @description Method that executes the token command `TransferOwnership`.
     * @see {BaseCommand.execute}
     **/
    execute(actor, argv) {
        // verify authorization to execute
        super.assertExecutionAllowance(actor, argv);
        // create the blockchain contract
        const contract = this.prepare();
        // return result
        return new symbol_uri_scheme_1.TransactionURI(contract.serialize());
    }
    /**
     * @description Wrap the command's transactions inside an aggregate transaction.
     * @see {BaseCommand.wrap}
     * @return {AggregateTransaction[]} Aggregate bonded transaction
     **/
    prepare() {
        if (!this.transactions.length) {
            throw new FailureEmptyContract_1.FailureEmptyContract('No transactions result of this contract execution.');
        }
        // create aggregate bonded
        return symbol_sdk_1.AggregateTransaction.createBonded(this.context.parameters.deadline, this.transactions, this.context.network.networkType, [], this.context.parameters.maxFee);
    }
}
exports.AbstractCommand = AbstractCommand;
