"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
/**
 * Copyright 2020 NEM
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
const symbol_sdk_1 = require("symbol-sdk");
// internal dependencies
const AbstractCommand_1 = require("./AbstractCommand");
/**
 * @class NIP13.AttachDocument
 * @package NIP13 Token Commands
 * @since v0.5.0
 * @description Class that describes a token command for attaching documents to NIP13 compliant tokens or to token partitions.
 * @summary This token command prepares one aggregate bonded transaction with following inner transactions:
 *
 *  - Transaction 01: Execution proof transaction
 *  - Transaction 02: Attach document hash to be signed by recipient account
 */
class AttachDocument extends AbstractCommand_1.AbstractCommand {
    constructor() {
        super(...arguments);
        /**
         * @description List of **required** arguments for this token command.
         */
        this.arguments = [
            'filenode',
            'filename',
            'recipient',
        ];
        // end-region abstract methods
    }
    // region abstract methods
    /**
     * @description Getter for the command name.
     * @see {BaseCommand.name}
     **/
    get name() {
        return 'AttachDocument';
    }
    /**
     * Getter for the command descriptor.
     *
     * @return {string}
     **/
    get descriptor() {
        return 'NIP13(v' + this.context.revision + ')' + ':document:' + this.identifier.id;
    }
    /**
     * @description Builds the inner transactions necessary for the
     *              execution of a `AttachDocument` command.
     * @see {AbstractCommand.transactions}
     * @return {Transaction[]} Aggregate bonded transaction
     **/
    get transactions() {
        // read external arguments
        const filenode = this.context.getInput('filenode', '');
        const filename = this.context.getInput('filename', '');
        const recipient = this.context.getInput('recipient', new symbol_sdk_1.PublicAccount());
        if (!filenode.length || !filename.length) {
            // Error: Invalid arguments
            return [];
        }
        if (!filenode.startsWith('Qm')) {
            // Error: Invalid IPFS file hash
            return [];
        }
        if (!recipient.address.equals(this.target.address)) {
            const the_partition = this.partitions.find((p) => p.account.address.equals(recipient.address));
            // AttachDocument is only possible for target account or existing partitions
            if (undefined === the_partition) {
                // Error: partition does not exist
                return [];
            }
        }
        // prepare output
        const transactions = [];
        const signers = [];
        // Transaction 01: Add execution proof transaction
        transactions.push(symbol_sdk_1.TransferTransaction.create(this.context.parameters.deadline, recipient.address, [], symbol_sdk_1.PlainMessage.create(this.descriptor), this.context.network.networkType, undefined));
        // Transaction 01 is issued by **recipient** account
        signers.push(recipient);
        // Transaction 02: Attach document hash to be signed by recipient account
        transactions.push(symbol_sdk_1.TransferTransaction.create(this.context.parameters.deadline, recipient.address, [], symbol_sdk_1.PlainMessage.create(filename + ':' + filenode), this.context.network.networkType, undefined));
        // Transaction 02 is issued by **recipient** account
        signers.push(recipient);
        // return transactions issued by assigned signer
        return transactions.map((transaction, i) => transaction.toAggregate(signers[i]));
    }
}
exports.AttachDocument = AttachDocument;
