"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
/**
 * Copyright 2020 NEM
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
const symbol_sdk_1 = require("symbol-sdk");
// internal dependencies
const TransferOwnership_1 = require("./TransferOwnership");
/**
 * @class NIP13.BatchTransferOwnership
 * @package NIP13 Token Commands
 * @since v0.1.0
 * @description Class that describes a token command for batch transferring NIP13 compliant tokens.
 * @summary
 * This token command accepts the following arguments:
 *
 * | Argument | Description | Example |
 * | --- | --- | --- |
 * | sender | Sender token holder partition account | `new PublicAccount(...)` |
 * | recipients | Recipient token holder partition accounts | `[new PublicAccount(...)]` |
 * | amount | Number of shares to be transferred | `1` |
 */
class BatchTransferOwnership extends TransferOwnership_1.TransferOwnership {
    constructor() {
        super(...arguments);
        /**
         * @description List of **required** arguments for this token command.
         */
        this.arguments = [
            'sender',
            'recipients',
            'amount',
        ];
        // end-region abstract methods
    }
    // region abstract methods
    /**
     * @description Getter for the command name.
     * @see {BaseCommand.name}
     **/
    get name() {
        return 'BatchTransferOwnership';
    }
    /**
     * Getter for the command descriptor.
     *
     * @return {string}
     **/
    get batchDescriptor() {
        return 'NIP13(v' + this.context.revision + ')' + ':batch:' + this.identifier.id;
    }
    /**
     * @description Builds the inner transactions necessary for the
     *              execution of a `BatchTransferOwnership` command.
     * @see {AbstractCommand.transactions}
     * @return {Transaction[]} Aggregate bonded transaction
     **/
    get transactions() {
        // read external arguments
        const recipients = this.context.getInput('recipients', []);
        // prepare output
        const transactions = [];
        const signers = [];
        // Transaction 01: Add execution proof transaction
        transactions.push(symbol_sdk_1.TransferTransaction.create(this.context.parameters.deadline, this.target.address, [], symbol_sdk_1.PlainMessage.create(this.batchDescriptor), this.context.network.networkType, undefined));
        // Transaction 01 is issued by **target** account
        signers.push(this.target);
        // :warning: Each partition listed in `recipients`
        // :warning: will produce the execution of one `TransferOwnership`
        let parentTransactions = [];
        for (let i = 0, m = recipients.length; i < m; i++) {
            const the_partition = recipients[i];
            // set `TransferOwnership` command arguments
            this.context.setInput('recipient', the_partition);
            // @see TransferOwnership.transactions()
            parentTransactions = parentTransactions.concat(super.transactions);
        }
        // return transactions issued by assigned signer
        return transactions.map((transaction, i) => transaction.toAggregate(signers[i])).concat(parentTransactions);
    }
}
exports.BatchTransferOwnership = BatchTransferOwnership;
