"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
/**
 * Copyright 2020 NEM
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
const symbol_sdk_1 = require("symbol-sdk");
// internal dependencies
const AbstractCommand_1 = require("./AbstractCommand");
/**
 * @class NIP13.ModifyRestriction
 * @package NIP13 Token Commands
 * @since v0.5.0
 * @description Class that describes a token command for modifying restrictions of NIP13 compliant tokens.
 * @summary
 * This token command accepts the following arguments:
 *
 * | Argument | Description | Example |
 * | --- | --- | --- |
 * | `restrictee` | Target account or partition account ("target of the restriction") | `new PublicAccount(...)` |
 * | `field` | Name of the restriction | `new PublicAccount(...)` |
 * | `value` | Minimumv value required for said restriction | `1` |
 */
class ModifyRestriction extends AbstractCommand_1.AbstractCommand {
    constructor() {
        super(...arguments);
        /**
         * @description List of **required** arguments for this token command.
         */
        this.arguments = [
            'restrictee',
            'field',
            'value',
        ];
        // end-region abstract methods
    }
    // region abstract methods
    /**
     * @description Getter for the command name.
     * @see {BaseCommand.name}
     **/
    get name() {
        return 'ModifyRestriction';
    }
    /**
     * Getter for the command descriptor.
     *
     * @return {string}
     **/
    get descriptor() {
        return 'NIP13(v' + this.context.revision + ')' + ':restriction:' + this.identifier.id;
    }
    /**
     * @description Builds the inner transactions necessary for the
     *              execution of a `ModifyRestriction` command.
     * @see {AbstractCommand.transactions}
     * @return {Transaction[]} Aggregate bonded transaction
     **/
    get transactions() {
        // read external arguments
        const restrictee = this.context.getInput('restrictee', new symbol_sdk_1.PublicAccount());
        const field = this.context.getInput('field', '');
        const value = this.context.getInput('value', 0);
        // prepare output
        const transactions = [];
        const signers = [];
        // Transaction 0.01: Execution proof transaction
        transactions.push(symbol_sdk_1.TransferTransaction.create(this.context.parameters.deadline, restrictee.address, [], symbol_sdk_1.PlainMessage.create(this.descriptor), this.context.network.networkType, undefined));
        // Transaction 0.01 is issued by **target** account
        signers.push(this.target);
        // 1) Modify **target** restrictions (MosaicGlobalRestriction)
        // :warning: Executing this command may render the affected
        //           token incompliant with NIP13 requirements.
        // :warning: Please, use this token command with care.
        if (restrictee.address.equals(this.target.address)) {
            const previousType = field === 'User_Role'
                ? symbol_sdk_1.MosaicRestrictionType.LE
                : symbol_sdk_1.MosaicRestrictionType.NONE;
            const previousValue = field === 'User_Role'
                ? 3 // 3 = default minimum value of "User_Role"
                : 0; // no previous value
            // Transaction 02: MosaicGlobalRestriction with mosaicId (refId 0)
            transactions.push(symbol_sdk_1.MosaicGlobalRestrictionTransaction.create(this.context.parameters.deadline, this.identifier.toMosaicId(), symbol_sdk_1.KeyGenerator.generateUInt64Key(field), symbol_sdk_1.UInt64.fromUint(previousValue), // previousRestrictionValue
            previousType, // previousRestrictionType
            symbol_sdk_1.UInt64.fromUint(value), // newRestrictionValue: 1 = Target ; 2 = Holder ; 3 = Locker ; 4 = Guest
            symbol_sdk_1.MosaicRestrictionType.LE, // newRestrictionType: `less or equal to`
            this.context.network.networkType, undefined, // referenceMosaicId: empty means "self"
            undefined));
            // Transaction 02 is issued by **target** account (multisig)
            signers.push(this.target);
        }
        // 2) Modify **partition** restrictions (MosaicAddressRestriction)
        else {
            // find partition
            const the_partition = this.partitions.find(p => p.account.address.equals(restrictee.address));
            // 'ModifyRestriction' is only possible for existing partitions
            if (undefined === the_partition) {
                // the partition doesn't exist
                return [];
            }
            // Transaction 02: MosaicAddressRestriction for account address
            transactions.push(symbol_sdk_1.MosaicAddressRestrictionTransaction.create(this.context.parameters.deadline, this.identifier.toMosaicId(), symbol_sdk_1.KeyGenerator.generateUInt64Key(field), the_partition.account.address, symbol_sdk_1.UInt64.fromUint(value), // newRestrictionValue
            this.context.network.networkType, symbol_sdk_1.UInt64.fromUint(2), // previousRestrictionValue: 2 = Holder
            undefined));
            // Transaction 02 is issued by the target account
            signers.push(this.target);
        }
        // return transactions issued by assigned signer
        return transactions.map((transaction, i) => transaction.toAggregate(signers[i]));
    }
}
exports.ModifyRestriction = ModifyRestriction;
