"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
/**
 * Copyright 2020 NEM
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
const symbol_sdk_1 = require("symbol-sdk");
const symbol_hd_wallets_1 = require("symbol-hd-wallets");
const HDService_1 = require("../services/HDService");
/**
 * @class Accountable
 * @package NIP13/contracts
 * @since v0.3.0
 * @description Class that describes an object able to create accounts.
 */
class Accountable {
    /**
     * Construct an `Accountable` object around \a bip39 mnemonic pass phrase.
     *
     * @param {MnemonicPassPhrase} bip39
     */
    constructor(
    /**
     * @description The network configuration
     */
    network, 
    /**
     * @description The BIP39 mnemonic pass phrase used with said token.
     */
    bip39) {
        this.network = network;
        this.bip39 = bip39;
        this.keyProvider = new symbol_hd_wallets_1.Wallet(symbol_hd_wallets_1.ExtendedKey.createFromSeed(this.bip39.toSeed().toString('hex'), symbol_hd_wallets_1.Network.CATAPULT));
    }
    /**
     * Derive a child account
     *
     * @param {string}    derivationPath
     * @return {Account}
     */
    getAccount(derivationPath) {
        return this.keyProvider.getChildAccount(derivationPath, this.network.networkType);
    }
    /**
     * Derive an **authority** account
     *
     * @return {Account}
     */
    getAuthority() {
        return this.getAccount(HDService_1.Derivation.AUTH_NIP13);
    }
    /**
     * Derive a **target** account
     *
     * @return {Account}
     */
    getTarget() {
        return this.getAccount(HDService_1.Derivation.PATH_NIP13);
    }
    /**
     * Derive an **operator** account
     *
     * @internal Operator account derivation paths use the REMOTE (BIP44=CHANGE) path level.
     * @param {number}  at
     * @return {Account}
     */
    getOperator(at = 1) {
        // prepare derivation
        const start = HDService_1.Derivation.PATH_NIP13;
        const level = HDService_1.Derivation.DerivationPathLevels.Remote;
        // derive operator
        const path = HDService_1.Derivation.HDService.incrementPathLevel(start, level, at);
        return this.getAccount(path);
    }
    /**
     * Derive a **partition** account
     *
     * @internal Partition account derivation paths use the ADDRESS path level.
     * @param   {PublicAccount}  owner
     * @param   {number}         at
     * @param   {string}         name
     * @return  {Account}
     */
    getPartition(owner, name = 'default') {
        const path = this.getPathForPartition(owner, name);
        return this.getAccount(path);
    }
    /**
     * Get the BIP44 path for a partition owned by `owner`
     *
     * @internal Partition account derivation paths use the ADDRESS path level.
     * @param   {PublicAccount} owner
     * @param   {number}         at
     * @return  {string}
     */
    getPathForPartition(owner, name = 'default') {
        // prepare derivation
        const start = HDService_1.Derivation.PATH_NIP13;
        const level = HDService_1.Derivation.DerivationPathLevels.Address;
        // prepare deterministic
        const hash = new Uint8Array(64);
        const data = name + '-' + owner.address.plain();
        symbol_sdk_1.SHA3Hasher.func(hash, symbol_sdk_1.Convert.utf8ToUint8(data), 64);
        // 3 right-most bytes for partition id
        // :warning: cannot be more than 3 bytes because of hardened
        //           derivation index overflow at 2147483647 (2^31-1)
        const right3b = parseInt(hash.slice(-3).reduce((s, b) => s + b.toString(16).padStart(2, '0'), ''), 16);
        // derive partition
        return HDService_1.Derivation.HDService.incrementPathLevel(start, level, right3b);
    }
}
exports.Accountable = Accountable;
