/**
 * Copyright 2020 NEM Foundation (https://nem.io)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
import { Service } from '../../../../index';
/**
 * @namespace Derivation
 * @package NIP13 services
 * @since v0.5.0
 * @description Namespace that encapsulates HD keys derivation features.
 */
export declare namespace Derivation {
    /**
     * @description The authorities accounts' derivation paths for NIP13
     */
    const AUTH_NIP13: string;
    /**
     * @description The target account's derivation path for NIP13
     */
    const PATH_NIP13: string;
    /**
     * @enum DerivationPathLevels
     * @description Enumeration of available derivation path levels (as of BIP44).
     * @link https://github.com/bitcoin/bips/blob/master/bip-0044.mediawiki#path-levels
     */
    enum DerivationPathLevels {
        Purpose = 1,
        CoinType = 2,
        Account = 3,
        Remote = 4,
        Address = 5
    }
    /**
     * @class MosaicService
     * @package services
     * @since v0.1.0
     * @description Class that describes a service around token metadata features.
     */
    class HDService extends Service {
        /**
         * @function NIP13.HDService.isValidPath()
         * @description Helper function to validate a derivation path.
         */
        static isValidPath: (path: string) => boolean;
        /**
         * @function NIP13.HDService.assertValidPath()
         * @description Helper function to assert whether a path is a valid derivation path.
         * @throws {FailureInvalidDerivationPath} On invalid derivation path provided.
         */
        static assertValidPath: (path: string) => void;
        /**
         * @function NIP13.HDService.assertCanModifyLevel()
         * @description Helper function to assert whether a derivation path level can be modified.
         * @throws {FailureInvalidDerivationPath} On invalid derivation path level provided.
         */
        static assertCanModifyLevel: (which: DerivationPathLevels) => void;
        /**
         * @function NIP13.HDService.incrementPathLevel()
         * @description Helper function to increment a derivation path level.
         * @throws {FailureInvalidDerivationPath} On invalid derivation path or derivation path level provided.
         */
        static incrementPathLevel: (path: string, which?: DerivationPathLevels, step?: number) => string;
        /**
         * @function NIP13.HDService.decrementPathLevel()
         * @description Helper function to decrement a derivation path level.
         * @throws {FailureInvalidDerivationPath} On invalid derivation path or derivation path level provided.
         */
        static decrementPathLevel: (path: string, which?: DerivationPathLevels, step?: number) => string;
        /**
         * @function NIP13.HDService.decrementPathLevel()
         * @description Helper function to decrement a derivation path level.
         * @throws {FailureInvalidDerivationPath} On invalid derivation path or derivation path level provided.
         */
        static getPaths: (startPath: string, size?: number) => string[];
    }
}
