"use strict";
/*
 * Copyright 2019-present NEM Foundation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
Object.defineProperty(exports, "__esModule", { value: true });
const rxjs_1 = require("rxjs");
const operators_1 = require("rxjs/operators");
const symbol_sdk_1 = require("symbol-sdk");
class TransactionService {
    constructor(accountRepository, chainRepository, transactionHttp, receiptHttp, pageSize) {
        this.accountRepository = accountRepository;
        this.chainRepository = chainRepository;
        this.transactionHttp = transactionHttp;
        this.receiptHttp = receiptHttp;
        this.pageSize = pageSize;
        this.transactionService = new symbol_sdk_1.TransactionService(transactionHttp, receiptHttp);
    }
    /* start block flattenAggregateTransactions */
    /**
     * Processes the list of transactions. In case there is an aggregate transaction,
     * removes the envelop returning separately each inner transaction.
     *
     * @param transactions List of transactions.
     * @returns Transaction[]
     */
    static flattenAggregateTransactions(transactions) {
        const txes = [];
        return txes.concat(transactions.map((transaction) => {
            let flattenTransactions;
            if (transaction instanceof symbol_sdk_1.AggregateTransaction) {
                flattenTransactions = transaction.innerTransactions;
            }
            else {
                flattenTransactions = [transaction];
            }
            return TransactionService.filterTransferTransactions(flattenTransactions);
        }).reduce((prev, it) => prev.concat(it), []));
    }
    /* end block flattenAggregateTransactions */
    /* start block filterTransferTransactions */
    /**
     * Given an array of transactions, returns the ones with type TRANSFER.
     *
     * @param transactions List of transactions.
     * @returns TransferTransaction[]
     */
    static filterTransferTransactions(transactions) {
        return transactions
            .filter((transaction) => (transaction.type === symbol_sdk_1.TransactionType.TRANSFER))
            .map((transaction) => transaction);
    }
    /* end block filterTransferTransactions */
    /* start block getTransactionHash */
    /**
     * Gets the transaction hash.
     * If it is an aggregate transaction, returns an aggregate transaction hash.
     *
     * @param transaction Transaction.
     * @returns string | undefined
     */
    static getTransactionHash(transaction) {
        const transactionInfo = transaction.transactionInfo;
        let hash;
        if (transactionInfo instanceof symbol_sdk_1.AggregateTransactionInfo) {
            hash = transactionInfo.aggregateHash;
        }
        else if (transactionInfo instanceof symbol_sdk_1.TransactionInfo) {
            hash = transactionInfo.hash;
        }
        return hash;
    }
    /* end block getTransactionHash */
    /* start block filterElligibleIncomingTransfers */
    /**
     * Given an array of transactions, returns the ones that are eligible to be _transfers_.
     * A transaction is an eligible deposit when:
     * - It has type TRANSFER.
     * - The recipient address is the exchange address.
     * - Contains tokenId's units.
     * - Does not contain mosaics different to tokenId.
     * - The message field is not empty.
     *
     * @param transactions Array of transactions that could be eligible _transfers_.
     * @param exchangeAddress Exchange central account address.
     * @param tokenId (Optional) Mosaic identifier to filter.
     * @returns TransferTransaction[]
     */
    static filterElligibleIncomingTransfers(transactions, exchangeAddress, tokenId) {
        return TransactionService.filterTransferTransactions(transactions)
            .filter((transaction) => (transaction.recipientAddress instanceof symbol_sdk_1.Address
            && transaction.recipientAddress.plain() === exchangeAddress.plain()))
            .filter((transaction) => (transaction.message instanceof symbol_sdk_1.PlainMessage
            && transaction.message.payload.length !== 0));
    }
    /* end block filterElligibleIncomingTransfers */
    /* start block filterElligibleOutgoingTransfers */
    /**
     * Given an array of transactions, returns the ones that are eligible to be _transfers_.
     * A transaction is an eligible deposit when:
     * - It has type TRANSFER.
     * - The recipient address is the exchange address.
     * - Contains tokenId's units.
     * - Does not contain mosaics different to tokenId.
     * - The message field is not empty.
     *
     * @param transactions Array of transactions that could be eligible _transfers_.
     * @param exchangeAddress Exchange central account address.
     * @param tokenId (Optional) Mosaic identifier to filter.
     * @returns TransferTransaction[]
     */
    static filterElligibleOutgoingTransfers(transactions, exchangeAddress, tokenId) {
        return TransactionService.filterTransferTransactions(transactions)
            .filter((transaction) => (transaction.signer.address.equals(exchangeAddress)))
            .filter((transaction) => (transaction.message instanceof symbol_sdk_1.PlainMessage
            && transaction.message.payload.length !== 0));
    }
    /* end block filterElligibleOutgoingTransfers */
    /* start block filterElligibleDeposits */
    /**
     * Given an array of transactions, returns the ones that are eligible to be deposits.
     * A transaction is an eligible deposit when:
     * - It has type TRANSFER.
     * - The recipient address is the exchange address.
     * - Contains tokenId's units.
     * - Does not contain mosaics different to tokenId.
     * - The message field is not empty.
     *
     * @param transactions Array of transactions that could be eligible deposits.
     * @param exchangeAddress Exchange central account address.
     * @param tokenId Mosaic identifier to filter.
     * @returns TransferTransaction[]
     */
    static filterElligibleDeposits(transactions, exchangeAddress, tokenId) {
        return TransactionService.filterTransferTransactions(transactions)
            .filter((transaction) => (transaction.recipientAddress instanceof symbol_sdk_1.Address
            && transaction.recipientAddress.equals(exchangeAddress)))
            .filter((transaction) => (transaction.mosaics.length === 1
            && transaction.mosaics[0].id instanceof symbol_sdk_1.MosaicId
            && transaction.mosaics[0].id.equals(tokenId)))
            .filter((transaction) => (transaction.message instanceof symbol_sdk_1.PlainMessage
            && transaction.message.payload.length !== 0));
    }
    /* end block filterElligibleDeposits */
    /* start block filterElligibleWithdrawals */
    /**
     * Given an array of transactions, returns the ones that are eligible to be deposits.
     * A transaction is an eligible deposit when:
     * - It has type TRANSFER.
     * - The signer public key is the exchange address.
     * - Contains tokenId's units.
     * - Does not contain mosaics different to tokenId.
     * - The message field is not empty.
     *
     * @param transactions Array of transactions that could be eligible deposits.
     * @param exchangeAddress Exchange central account address.
     * @param tokenId Mosaic identifier to filter.
     * @returns TransferTransaction[]
     */
    static filterElligibleWithdrawals(transactions, exchangeAddress, tokenId) {
        return TransactionService.filterTransferTransactions(transactions)
            .filter((transaction) => (transaction.signer.address.equals(exchangeAddress)))
            .filter((transaction) => (transaction.mosaics.length === 1
            && transaction.mosaics[0].id instanceof symbol_sdk_1.MosaicId
            && transaction.mosaics[0].id.equals(tokenId)))
            .filter((transaction) => (transaction.message instanceof symbol_sdk_1.PlainMessage
            && transaction.message.payload.length !== 0));
    }
    /* end block filterElligibleWithdrawals */
    /* start block getIncomingTransfers */
    /**
     * Gets the transactions eligible to be deposits pending to be processed.
     * @param exchangeAddress Exchange central account address.
     * @param tokenId Mosaic identifier.
     * @param requiredConfirmations Confirmations to consider a transaction persistent.
     * @param lastTransactionId Resource identifier of the last transaction already processed.
     * @returns Observable <TransferTransaction[]>
     */
    getIncomingTransfers(exchangeAddress, tokenId, requiredConfirmations, lastTransactionId) {
        return this.getUnprocessedTransactions(exchangeAddress, lastTransactionId).pipe(operators_1.mergeMap((transactions) => this.resolveTransactionsAliases(transactions)), operators_1.map((transactions) => TransactionService.flattenAggregateTransactions(transactions)), operators_1.mergeMap((transactions) => this.filterTransactionsWithEnoughConfirmations(transactions, requiredConfirmations)), operators_1.map((transactions) => TransactionService.filterElligibleIncomingTransfers(transactions, exchangeAddress, tokenId)));
    }
    /* end block getIncomingTransfers */
    /* start block getOutgoingTransfers */
    /**
     * Gets the transactions eligible to be deposits pending to be processed.
     * @param exchangeAddress Exchange central account address.
     * @param tokenId Mosaic identifier.
     * @param requiredConfirmations Confirmations to consider a transaction persistent.
     * @param lastTransactionId Resource identifier of the last transaction already processed.
     * @returns Observable <TransferTransaction[]>
     */
    getOutgoingTransfers(exchangeAddress, tokenId, requiredConfirmations, lastTransactionId) {
        return this.getUnprocessedTransactions(exchangeAddress, lastTransactionId).pipe(operators_1.mergeMap((transactions) => this.resolveTransactionsAliases(transactions)), operators_1.map((transactions) => TransactionService.flattenAggregateTransactions(transactions)), operators_1.mergeMap((transactions) => this.filterTransactionsWithEnoughConfirmations(transactions, requiredConfirmations)), operators_1.map((transactions) => TransactionService.filterElligibleOutgoingTransfers(transactions, exchangeAddress, tokenId)));
    }
    /* end block getOutgoingTransfers */
    /* start block getDeposits */
    /**
     * Gets the transactions eligible to be deposits pending to be processed.
     * @param exchangeAddress Exchange central account address.
     * @param tokenId Mosaic identifier.
     * @param requiredConfirmations Confirmations to consider a transaction persistent.
     * @param lastTransactionId Resource identifier of the last transaction already processed.
     * @returns Observable <TransferTransaction[]>
     */
    getDeposits(exchangeAddress, tokenId, requiredConfirmations, lastTransactionId) {
        return this.getUnprocessedTransactions(exchangeAddress, lastTransactionId).pipe(operators_1.mergeMap((transactions) => this.resolveTransactionsAliases(transactions)), operators_1.map((transactions) => TransactionService.flattenAggregateTransactions(transactions)), operators_1.mergeMap((transactions) => this.filterTransactionsWithEnoughConfirmations(transactions, requiredConfirmations)), operators_1.map((transactions) => TransactionService.filterElligibleDeposits(transactions, exchangeAddress, tokenId)));
    }
    /* end block getDeposits */
    /* start block getWithdrawals */
    /**
     * Gets the transactions eligible to be withdrawals pending to be processed.
     * @param exchangeAddress Exchange central account address.
     * @param tokenId Mosaic identifier.
     * @param requiredConfirmations Confirmations to consider a transaction persistent.
     * @param lastTransactionId Resource identifier of the last transaction already processed.
     * @returns Observable <TransferTransaction[]>
     */
    getWithdrawals(exchangeAddress, tokenId, requiredConfirmations, lastTransactionId) {
        return this.getUnprocessedTransactions(exchangeAddress, lastTransactionId).pipe(operators_1.mergeMap((transactions) => this.resolveTransactionsAliases(transactions)), operators_1.map((transactions) => TransactionService.flattenAggregateTransactions(transactions)), operators_1.mergeMap((transactions) => this.filterTransactionsWithEnoughConfirmations(transactions, requiredConfirmations)), operators_1.map((transactions) => TransactionService.filterElligibleWithdrawals(transactions, exchangeAddress, tokenId)));
    }
    /* end block getWithdrawals */
    /* start block getUnprocessedTransactions */
    /**
     * Gets the list of transactions pending to be processed.
     * @param exchangeAddress Exchange central account address.
     * @param lastTransactionId Resource identifier of the last transaction already processed.
     * @returns Observable <Transaction[]>
     */
    getUnprocessedTransactions(exchangeAddress, lastTransactionId) {
        const queryParams = new symbol_sdk_1.QueryParams({ pageSize: this.pageSize, order: symbol_sdk_1.Order.ASC, id: lastTransactionId });
        // Filter transfer and aggregate transactions
        const transactionFilter = new symbol_sdk_1.TransactionFilter({ types: [
                symbol_sdk_1.TransactionType.TRANSFER,
                symbol_sdk_1.TransactionType.AGGREGATE_COMPLETE,
                symbol_sdk_1.TransactionType.AGGREGATE_BONDED
            ] });
        return this.accountRepository
            .getAccountTransactions(exchangeAddress, queryParams, transactionFilter).pipe(
        // Get all transactions pending to be processed
        operators_1.expand((transactions) => {
            if (transactions.length === this.pageSize) {
                return this.accountRepository.getAccountTransactions(exchangeAddress, new symbol_sdk_1.QueryParams({ pageSize: this.pageSize,
                    order: symbol_sdk_1.Order.ASC,
                    id: transactions[transactions.length - 1].transactionInfo.id }), transactionFilter);
            }
            return rxjs_1.EMPTY;
        }), operators_1.concatMap((_) => _), operators_1.toArray());
    }
    /* end block getUnprocessedTransactions */
    /* start block filterTransactionsWithEnoughConfirmations */
    /**
     * Gets deposits with enough confirmations.
     * @param transactions Array of transactions to check.
     * @param requiredConfirmations Number of confirmations to consider a transaction valid.
     * @returns Observable <Transaction[]>
     */
    filterTransactionsWithEnoughConfirmations(transactions, requiredConfirmations) {
        return this.chainRepository.getBlockchainHeight().pipe(
        // Determine if the transactions have received enough confirmations.
        operators_1.map((currentHeight) => transactions.filter((transaction) => {
            const transactionHeight = transaction.transactionInfo.height;
            return (currentHeight.subtract(transactionHeight)
                .compare(symbol_sdk_1.UInt64.fromUint(requiredConfirmations)) >= 0);
        })));
    }
    /* end block filterTransactionsWithEnoughConfirmations */
    /* start block resolveTransactionsAliases */
    /**
     * Resolves the aliases behind an announced transaction.
     * @param transactions Array of transactions to resolve.
     * @returns Observable <Transaction[]>
     */
    resolveTransactionsAliases(transactions) {
        const transactionHashes = transactions.map(((transaction) => transaction.transactionInfo.hash));
        return this.transactionService.resolveAliases(transactionHashes);
    }
}
exports.TransactionService = TransactionService;
